/**
 * Gestion des messages de debogage dans la console
 * @note Implémenter cette interface pour toutes les classes à debugguer
 * Dans le constructeur, utiliser super(true) pour debugger la classe, super(false) sinon.
 */

export interface IDebug {

    readonly DBG: boolean;

    debug(s: any): void;

}

// tslint:disable-next-line:max-classes-per-file
export class Debug {
    /**
     * @param _DBG Flag de débuggage
     */
    constructor(private _DBG: boolean) { }

    /**
     * Affiche un message dans la console si le flag this.DBG est à true
     * @param s Message à afficher dans la console
     */
    public debug(...args: any[]) {
        // tslint:disable-next-line:no-console
        if (this._DBG) {
            console.log(...args);
        }
    }

    get DBG() { return this._DBG; }
}

/**
 * Méthode simulant l'opérateur booléen xor
 * @see http://www.howtocreate.co.uk/xor.html
 */
export function XOR(a: boolean, b: boolean): boolean {
    return (a || b) && !(a && b);
}

/**
 * Méthode simulant l'opérateur booléen identité (vrai si a=b)
 */
export function BoolIdentity(a: boolean, b: boolean): boolean {
    return (a && b) || (!a && !b);
}

/**
 * arrondi d'un nombre avec une précision donnée
 * @param val nombre à arrondir
 * @param prec nombre de chiffres
 */
export function round(val: number, prec: number) {
    const m = Math.pow(10, prec || 0);
    return Math.round(val * m) / m;
}
