import { ParamDefinition } from "../param/param-definition";
import { ParamDomainValue } from "../param/param-domain";
import { ParallelStructureParams } from "./parallel_structure_params";

/**
 * Common parameters of hydraulic structure equations
 */
export class CloisonsParams extends ParallelStructureParams {
    /** Largeur des bassins (m) */
    public BB: ParamDefinition;

    /** Longueur des bassins (m) */
    public LB: ParamDefinition;

    /** Profondeur moyenne (m) */
    public PB: ParamDefinition;

    /** Hauteur de chute (m) */
    public DH: ParamDefinition;

    /**
     * Paramètres communs à toutes les équations de structure
     * @param rQ Débit total (m3/s)
     * @param rZ1 Cote de l'eau amont (m)
     * @param rLB Longueur des bassins (m)
     * @param rBB Largeur des bassins (m)
     * @param rPB Profondeur moyenne (m)
     * @param rDH Hauteur de chute (m)
     */
    constructor(rQ: number, rZ1: number, rLB: number, rBB: number, rPB: number, rDH: number) {
        super(rQ, rZ1, rZ1 - rDH);
        this.LB = new ParamDefinition("LB", ParamDomainValue.POS, rLB);
        this.addParamDefinition(this.LB);
        this.BB = new ParamDefinition("BB", ParamDomainValue.POS, rBB);
        this.addParamDefinition(this.BB);
        this.PB = new ParamDefinition("PB", ParamDomainValue.POS, rPB);
        this.addParamDefinition(this.PB);
        this.DH = new ParamDefinition("DH", ParamDomainValue.POS, rDH);
        this.addParamDefinition(this.DH);
    }
}
