export enum StructureType {
    SeuilRectangulaire,
    VanneRectangulaire,
    Orifice,
    SeuilTriangulaire,
    SeuilTriangulaireTrunc
    // VanneCirculaire,
    // VanneTrapezoidale, SeuilTrapezoidal
}

export enum LoiDebit {
    // loi de débit Déversoir / Orifice Cemagref 1988
    Cem88d,
    // loi de débit Déversoir / Vanne de fond Cemagref 1988
    Cem88v,
    // loi de débit Cunge 1980
    Cunge80,
    // loi de débit pour vanne dénoyée
    RectangularOrificeFree,
    // loi de débit pour vanne noyée
    RectangularOrificeSubmerged,
    // loi de débit pour seuil dénoyé
    WeirFree,
    // Loi Kindsvater-Carter et Villemonte
    KIVI,
    // Loi de débit seuil triangulaire dénoyé
    TriangularWeirFree,
    // Loi de débit seuil triangulaire tronqué
    TriangularTruncWeirFree,
    // Loi de débit seuil noyé (Larinier 1992)
    WeirSubmergedLarinier,
    // Loi de débit orifice noyé
    OrificeSubmerged
}

export const loiAdmissibles: { [key: string]: LoiDebit[] } = {
    Orifice: [
        LoiDebit.OrificeSubmerged
    ],
    SeuilRectangulaire: [
        LoiDebit.Cem88d, LoiDebit.Cem88v, LoiDebit.Cunge80, LoiDebit.WeirFree,
        LoiDebit.KIVI, LoiDebit.WeirSubmergedLarinier
    ],
    SeuilTriangulaire: [
        LoiDebit.TriangularWeirFree
    ],
    SeuilTriangulaireTrunc: [
        LoiDebit.TriangularTruncWeirFree
    ],
    VanneRectangulaire: [
        LoiDebit.Cem88d, LoiDebit.Cem88v, LoiDebit.Cunge80, LoiDebit.RectangularOrificeFree,
        LoiDebit.RectangularOrificeSubmerged
    ]
};

export class StructureProperties {
    /**
     * @return true si les valeurs de StructureType et LoiDebit sont compatibles
     */
    public static isCompatibleValues(struct: StructureType, loi: LoiDebit): boolean {
        return loiAdmissibles[StructureType[struct]].includes(loi);
    }

    /**
     * @return la 1ère valeur de StructureType compatible avec la loi de débit
     */
    public static findCompatibleStructure(loi: LoiDebit): StructureType {
        for (const st in loiAdmissibles) {
            if (loiAdmissibles.hasOwnProperty(st)) {
                const lds: LoiDebit[] = loiAdmissibles[st];
                for (const ld of lds) {
                    if (ld === loi) { return (StructureType as any)[st]; }
                }
            }
        }
        return undefined;
    }

    /**
     * trouve la 1ère valeur de LoiDebit compatible avec le type de structure
     * @param struct type de structure avec laquelle la loi de débit doit être compatible
     * @param subset si non vide, recherche la loi de débit compatible dans ce tableau; sinon prend la 1ere
     */
    public static findCompatibleLoiDebit(struct: StructureType, subset: LoiDebit[]): LoiDebit {
        const sst: string = StructureType[struct];

        if (subset.length === 0) {
            return loiAdmissibles[sst][0];
        }

        for (const ld of loiAdmissibles[sst]) {
            if (subset.includes(ld)) {
                return ld;
            }
        }

        return undefined;
    }
}
