import { Message, MessageCode } from "./message";
import { Pair } from "./pair";

/**
 * couple de valeurs ordonnées
 */
export class Interval extends Pair {
    constructor(bound1: number, bound2: number) {
        super(bound1, bound2);
    }

    public checkValue(v: number) {
        if (v == undefined) {
            const e = new Message(MessageCode.ERROR_INTERVAL_UNDEF);
            throw e;
        }

        if (!this.intervalHasValue(v)) {
            const e = new Message(MessageCode.ERROR_INTERVAL_OUTSIDE);
            e.extraVar.value = v;
            e.extraVar.interval = this.toString();
            throw e;
        }
    }

    get length(): number {
        return this.max - this.min;
    }

    public intersect(i: Interval): Interval {
        const min = Math.max(this.min, i.min);
        const max = Math.min(this.max, i.max);
        return new Interval(min, max);
    }

    public setBounds(i: Pair) {
        this.setPair(i);
    }
}
